cfetApp.CFETModule.controller('MainController',
    function ($scope, $rootScope, cfService, $modal, $location) {
        $scope.availableLanguageIds = cfService.getAvailableLanguageIds();


        var i18n = {
            lng: $scope.availableLanguageIds[0]
        };

        $rootScope.i18n = i18n;
        $rootScope.brand = {
            id: 'castforward'
        };

        if (app.usersLanguageId) {
            if ($scope.availableLanguageIds.indexOf(app.usersLanguageId) >= 0) {
                i18n.lng = app.usersLanguageId;
            }
        }

        // die Sprache geht als Parameter ein, so dass Angular die Funktion bei der Sprach-Aenderung erneut aufruft
        i18n.getText = function (lng, lngKey) {
            var translated = cfetApp.translations[lng][lngKey];
            if (!translated) {
                translated = '???' + lngKey + '???';
            }

            // extra params
            for (var i = 2; i < arguments.length; i++) {
                translated = translated.replace(/%%/, arguments[i]);
            }

            return translated;
        };

        i18n.setLanguage = function (lngKey) {
            i18n.lng = lngKey;
        };


        $scope.languages = {};
        for (var i = 0; i < $scope.availableLanguageIds.length; i++) {
            $scope.languages[$scope.availableLanguageIds[i]] = i18n.getText(i18n.lng, $scope.availableLanguageIds[i]);
        }

        $rootScope.cfService = cfService;


        // offline and link tooling

        $scope.openWeb = function (webId, params) {
            console.log("show external " + webId);
            if (cfService.isOffline()) {
                $scope.showOfflineMessage('offline_page_cannot_be_opened');
                return;
            }

            // rueckfrage
            var modal = $modal.open({
                templateUrl: 'app/template/openExternalDialog.html',
                size: 'sm',
                scope: $scope,
                backdrop: true
            });

            modal.result.then(function() {
                successDoOpen();
            });


            function successDoOpen() {
                switch (webId) {
                    case WEB_MY_PROFILE:
                        cfService.openMyEProfile();
                        break;
                    case WEB_EPROFILE:
                        cfService.openEProfile(params.profileLink);
                        break;
                    case WEB_ESEARCH:
                        cfService.openESearch();
                        break;
                    case WEB_CASTINGLETTER:
                        cfService.openCastingLetter();
                        break;
                    case WEB_VOLLVERSION:
                        cfService.openFullVersion();
                        break;
                    case WEB_HELP:
                        cfService.openOnlineHelp();
                        break;
                    case WEB_ECASTING:
                        cfService.openECastings();
                        break;
                    case WEB_MY_PROJECTS:
                        cfService.openMyProjects();
                        break;
                }
            }
        };

        $scope.showOfflineMessage = function (msgkey) {
            if (!msgkey) {
                msgkey = 'device_is_offline';
            }
            $scope.msgHeaderKey = 'fehler';
            $scope.msgBodyKey = msgkey;
            $modal.open({
                templateUrl: 'app/template/alertDialog.html',
                size: 'sm',
                scope: $scope,
                backdrop: true
            });

            return;
        };

        $scope.openShowreel = function(profileId, title) {
            var url = '/showreel/0/x';
            if(profileId) {
                url = '/showreel/' + encodeURIComponent(profileId) + '/' + encodeURIComponent(title);
            } else {
                profileId = cfService.getUser().getProfileId();
            }
            navIfOnlineOrSynced(cfService.hasVideoListForProfileId(profileId), url, 'offline_and_no_videos_synced');
        };

        $scope.openPictures = function(profileId, title) {
            var url = '/bilder/0/x';
            if(profileId) {
                url = '/bilder/' + encodeURIComponent(profileId) + '/' + encodeURIComponent(title);
            } else {
                profileId = cfService.getUser().getProfileId();
            }
            navIfOnlineOrSynced(cfService.hasPictureListForProfileId(profileId), url, 'offline_and_no_pictures_synced');
        };

        $scope.openClients = function() {
            navIfOnlineOrSynced(cfService.hasClientList(), '/klienten', 'offline_and_no_clients_synced');
        };

        function navIfOnlineOrSynced(promise, url, offlineMsgKey) {
            promise.then(
                function() {
                    $location.path(url);
                },
                function() {
                    if(cfService.isOffline()) {
                        $scope.showOfflineMessage(offlineMsgKey);
                        return;
                    }

                    $location.path(url);
                });
        };
    });