function CFClient(config) {
    this.baseUrl = config.baseUrl;
    this.jsonRPCEndpoint = config.jsonRPCEndpoint;

    // angluars $http is expected
    this.httpClient = config.httpClient;
    // angualars $q is expected
    this.q = config.q;
    // angulars $cookies is expected
    this.cookies = config.cookies;

    this.userId = config.userId;
    this.password = config.password;
    this.xsrfToken = config.xsrfToken;
    this.sessionId = config.sessionId;
    this.remeberMeToken = config.remeberMetoken;
    this.rpcCount = 0;


    // expose for testing
    window.cfClient = this;

    return this;
}


CFClient.prototype.getOneTimeTicket = function() {
    var deferred = this.doRPC('getOneTimeTicket', {});

    return deferred.promise;
};

CFClient.prototype.authenticate = function (username, password, language) {
    var me = this;

    var url = this.baseUrl + this.jsonRPCEndpoint;

    // clear data
    this.userId = null;
    this.password = null;
    this.sessionId = null;
    this.password = null;

    var deferred = this.q.defer();
    var cancelOp = this.q.defer();

    var data = {
        id: ++this.rpcCount,
        jsonrpc: '2.0',
        method: 'doLogin',
        params: {
            username: username,
            password: password,
            language: language,
            rememberMe: true
        }
    };

    this.httpClient({
        method: 'POST',
        url: url,
        data: data,
        // never send existing tokens or sids on login, although the server has to make sure that there is a new session and xsrf token created on every login
        transformRequest: function(data, headersGetter) {
            var headers = headersGetter();
            delete headers[XSRF_HEADER];
            delete headers[SESSION_HEADER];

            return JSON.stringify(data);
        },
        xsrfHeaderName: XSRF_HEADER,
        xsrfCookieName: XSRF_COOKIE,
        timeout: cancelOp
    }).then(success, error);

    function success(response) {
        me.__handleLogin(response, deferred);
    }


    function error(response) {
        console.log("error", response);

        deferred.reject(response);
    }


    deferred.promise.cancel = function() {
        console.log("canceling login request");
        cancelOp.resolve();
    };
    return deferred.promise;
};

CFClient.prototype.getLoginStatus = function() {
    var me = this;

    var url = this.baseUrl + this.jsonRPCEndpoint;

    // clear data
    this.userId = null;
    this.password = null;
    this.sessionId = null;
    this.password = null;

    var deferred = this.q.defer();
    var cancelOp = this.q.defer();

    var data = {
        id: ++this.rpcCount,
        jsonrpc: '2.0',
        method: 'getLoginState',
        params: {}
    };

    this.httpClient({
        method: 'POST',
        url: url,
        data: data,
        // never send existing tokens or sids on login, although the server has to make sure that there is a new session and xsrf token created on every login
        transformRequest: function(data, headersGetter) {
            var headers = headersGetter();
            delete headers[XSRF_HEADER];
            delete headers[SESSION_HEADER];

            return JSON.stringify(data);
        },
        xsrfHeaderName: XSRF_HEADER,
        xsrfCookieName: XSRF_COOKIE,
        timeout: cancelOp
    }).then(success, error);

    function success(response) {
        me.__handleLogin(response, deferred);
    }

    function error(response) {
        console.log("error", response);

        deferred.reject(response);
    }


    deferred.promise.cancel = function() {
        console.log("canceling login request");
        cancelOp.resolve();
    };
    return deferred.promise;
};

CFClient.prototype.__handleLogin = function (response, deferred) {
    if(response.data.error != null) {
        deferred.reject("failed rpc");
        return;
    }

    // falsches Passwort oder username wird wie festgestellt?
    // feld login

    //TEST
    //response.data.result.login = false;
    if(response.data.result.login === false) {
        deferred.reject("failed login");
        return;
    }

    var user = new CFUser(response.data.result);

    var cookies = response.headers('Set-Cookie');
    if(cookies) {
        console.log(cookies);

        var sidMatch = cookies.match(/cfet_xas_sid=([^;]+);/i);
        var xsrfTokenMatch = cookies.match(/XSRF-TOKEN=([^;]+);/i);
        var remMatch = cookies.match(/cfet_xas_rem=([^;]+);/i);
        var sessionId = '';
        var xsrfToken = '';
        var remToken = '';
        if (sidMatch && sidMatch.length == 2) {
            sessionId = sidMatch[1];
        }
        if (xsrfTokenMatch && xsrfTokenMatch.length == 2) {
            xsrfToken = xsrfTokenMatch[1];
        }
        if (remMatch && remMatch.length == 2) {
            remToken = remMatch[1];
        }
    }

    deferred.resolve({user: user, sessionId: sessionId, xsrfToken: xsrfToken, remToken: remToken});
};

CFClient.prototype.getBaseUrl = function () {
    return this.baseUrl;
};

CFClient.prototype.setCredentials = function (userId, password) {
    this.userId = userId;
    this.password = password;
};

CFClient.prototype.setSessionId = function (sessionId) {
    this.sessionId = sessionId;
};

CFClient.prototype.setXSRFToken = function (xsrfToken) {
    this.xsrfToken = xsrfToken;
};

CFClient.prototype.setRememberMeToken = function (remToken) {
    this.remeberMeToken = remToken;
};



CFClient.prototype.getVideoListByProfileId = function(profileId) {
    var params = {
        profileId: profileId
    };

    var deferred = this.doRPC('appGetPlaylist', params);

    return deferred.promise;
};

CFClient.prototype.getPictureListByProfileId = function(profileId) {

    var params = {
        profileId: profileId
    };

    var deferred = this.doRPC('appGetFotos', params);

    return deferred.promise;
};

CFClient.prototype.getClientList = function() {

    var params = {
    };

    var deferred = this.doRPC('loadClientList', params);

    return deferred.promise;
};

CFClient.prototype.getLanguageUpdate = function(lngKey) {
    var me = this;
    var url = this.baseUrl + '/i18n/translations_' + lngKey + '.js';

    // cache buster
    url = url + "?cb=" + (new Date()).getTime();

    var deferred = this.q.defer();

    this.httpClient({
        method: 'GET',
        url: url
    }).then(success, error);

    function success(response) {

        deferred.resolve(response.data);
    }

    function error(response) {
        deferred.reject(response);
    }

    return deferred.promise;

};

CFClient.prototype.doRPC = function(method, params) {
    var me = this;
    var url = this.baseUrl + this.jsonRPCEndpoint;

    var deferred = this.q.defer();

    var data = {
        id: ++this.rpcCount,
        jsonrpc: '2.0',
        method: method,
        params: params
    };

    this.httpClient({
        method: 'POST',
        url: url,
        data: data,
        withCredentials: true,
        transformRequest: function(data, headersGetter) {
            var headers = headersGetter();
            if(me.sessionId) {
                headers[COOKIE_HEADER] = SESSION_COOKIE + '=' + me.sessionId + ';'
            }
            if(me.remeberMeToken) {
                headers[COOKIE_HEADER] = REMEBER_ME_COOKIE + '=' + me.remeberMeToken + ';'
            }
            if(me.xsrfToken) {
                headers[XSRF_HEADER] = me.xsrfToken;
            }

            return JSON.stringify(data);
        }
    }).then(success, error);

    function success(response) {
        if(response.data.error != null) {
            deferred.reject("failed rpc");
            return;
        }

        var result = response.data.result;

        deferred.resolve(result);
    }

    function error(response) {
        deferred.reject(response);
    }

    return deferred;
};
