function CFDownloadManager(config) {

    this.downloads = [];
    this.refIdCounter = new Date().getTime();
    this.trustAllHosts = true;
    this.q = config.q;
    this.useFS = new UseFS({q:config.q});
    this.downloadsEnabled = true;
    return this;
}

CFDownloadManager.prototype.queueDownload = function(config) {
    // erzeugt eine downloaderRefId


    config.refId = ++this.refIdCounter;
    config.tempFilePath = config.targetPath + ".partial-" + config.refId; // im selben verzeichnis
    config.dlStatus = 'AVAILABLE';
    config.dlPercent = 0;
    this.downloads.push(config);

    if(this.downloadsInProgress() == 0) {
        // okay es gab noch keinen, dann wird direkt losgelegt
        this.doDownload();
    }

    return config.refId;
};

CFDownloadManager.prototype.doDownload = function() {
    var me = this;

    if(!this.downloadsEnabled) {
        console.log("downloader disabled");
        return;
    }

    // finde den naechsten freien
    var config;
    for(var i = 0; i < this.downloads.length; i++) {
        if(this.downloads[i].dlStatus == 'AVAILABLE') {
            config = this.downloads[i];
            break;
        }
    }
    if(!config) {
        return;
    }

    console.log("starting download of", config);

    var ft = new FileTransfer();
    var options = config.options || {};
    config.dlStatus = 'STARTED';
    config.dlPercent = 'STARTED';
    config.fileTransfer = ft;

    ft.onprogress = onProgress;
    ft.download(config.sourceUrl, config.tempFilePath, success, fail, this.trustAllHosts, options);

    function onProgress(info) {
        // props
        // loaded, total
        config.bytesLoaded = info.loaded;
        config.bytesTotal = info.total;
        config.dlPercent = Math.floor(100 * info.loaded/info.total);
    }

    function success(fileEntry) {
        console.log("download completed", config, fileEntry);

        if(config.dlStatus == 'CANCELED') {
            console.log("canceled download completed successful");
            if(config.dlAbortCB) {
                config.dlAbortCB();
            }
            return;
        }




        //rename to target Path
        fileEntry.getParent(function(parentDirEntry) {
            var newName = UseFS.getName(config.targetPath);
            fileEntry.moveTo(parentDirEntry, newName, function(newEntry) {
                config.dlStatus = 'COMPLETED';
                // kick off next download
                me.doDownload();
                // notify short dl listeners
                if(config.defer) {
                    config.defer.resolve(newEntry);
                }
            }, fileError);
        }, fileError);


        function fileError(error) {
            console.log("fileop failed: ", error);
            config.dlStatus = 'FAILED';
            me.doDownload();
            if(config.defer) {
                config.defer.reject(error);
            }
        }

    }

    function fail(error) {

        if(error.code === FileTransferError.ABORT_ERR) {
            console.log("download canceled", error);
            config.dlStatus = 'CANCELED';
            if(config.dlAbortCB) {
                config.dlAbortCB();
            }
        } else {
            console.log("download failed", error);
            config.dlStatus = 'FAILED';
            // bei cancel wird abgebrochen
            me.doDownload();
        }
    }
};

CFDownloadManager.prototype.downloadsInProgress = function() {
    var count = 0;
    for(var i = 0; i < this.downloads.length; i++) {
        if(this.downloads[i].dlStatus === 'STARTED') {
            count++;
        }
    }
    return count;
};

CFDownloadManager.prototype.downloadsEnqueued = function() {
    var count = 0;
    for(var i = 0; i < this.downloads.length; i++) {
        if(this.downloads[i].dlStatus === 'STARTED' || this.downloads[i].dlStatus === 'AVAILABLE') {
            count++;
        }
    }
    return count;
};

CFDownloadManager.prototype.cancelDownloads = function(purge) {
    var me = this;
    var def = this.q.defer();

    this.downloadsEnabled = false;

    var idx = 0;
    function action() {
        if (idx == me.downloads.length) {

            if (purge) {
                me.downloads = [];
            }

            me.downloadsEnabled = true;
            def.resolve();

            return;
        }

        var dl = me.downloads[idx];
        idx++;
        if (dl.dlStatus == 'STARTED') {
            try {
                dl.dlStatus = 'CANCELED';

                delete dl.dlPercent;
                delete dl.bytesLoaded;
                delete dl.bytesTotal;

                if (dl.fileTransfer) {
                    dl.dlAbortCB = function() {
                        console.log("download aborted", dl);
                        // partial file loeschen
                        var fileUrl = dl.tempFilePath;
                        me.useFS.fs().useFile(fileUrl).unlink().then(function () {
                            console.log("removed canceled download: url=" + fileUrl);
                        }, function (error) {
                            console.log("failed to remove canceled download: url=" + fileUrl, error);
                        })['finally'](function() {
                            action();
                        });
                    };
                    console.log("about to cancel dl ", dl);
                    dl.fileTransfer.abort(); //ruft den fail callback der download-Methode auf
                } else {
                    action();
                }
            } catch (x) {
                console.log("error on download cancel", x);
                action();
            }
        } else {
            action();
        }
    }


    action();

    return def.promise;
};

CFDownloadManager.prototype.getDownloadStatus = function() {
    return this.downloads;
};



