/**
 * Service Facade for common tasks
 *
 */
function CFService(config) {
    this.user = null;
    this.baseUrl = config.baseUrl;
    this.cfClient = config.cfClient;
    // ngCordova file service expected
    this.fileService = config.fileService;
    this.availableLanguageIds = config.availableLanguageIds;
    // ngcordova device service expected
    this.deviceService = config.deviceService;
    // angular $q service
    this.q = config.q;

    this.offline = config.offline;
    //TEST
    //this.offline = true;

    this.useFS = new UseFS({q: this.q});

    // init storage
    this.dataDirUrl = ''; // wird von initStorage gesetzt
    this.privateDataDirUrl = ''; // wird von initPrivateStorage gesetzt
    this.initStorage();
    this.initPrivateStorage();

    this.videoDownloader = new CFDownloadManager({q: this.q});
    this.pictureDownloader = new CFDownloadManager({q: this.q});


    this.validScanLinkRegexes = config.validScanLinkRegexes;

    this.devMode = config.devMode === true;

    // expose for testing
    window.cfService = this;
    return this;
}


CFService.prototype.getUser = function () {
    return this.user;
};


CFService.prototype.setUser = function (user) {
    this.user = user;
};

CFService.prototype.isDevMode = function () {
    return this.devMode === true;
};


CFService.prototype.isValidScanLink = function (link) {
    if (!link) {
        return false;
    }

    // TODO: check valid urls from config list
    for (var i = 0; i < this.validScanLinkRegexes.length; i++) {
        if (this.validScanLinkRegexes[i].test(link)) {
            return true;
        }
    }

    return false;
};


CFService.prototype.openWebsiteURL = function (url) {
    console.log("open website url: " + url);
    window.open(url, '_system');
};

CFService.prototype.openWebsiteUrlWithAuthentication = function (url) {
    var me = this;
    this.cfClient.getOneTimeTicket().then(success, error);
    function success(ticket) {
        //members/ssoAuth.php?authToken={token}&redirectUrl={url}
        var ticketAuthenUrl = me.baseUrl + '/members/ssoAuth.php?authToken=' + encodeURIComponent(ticket) + '&redirectUrl=' + encodeURIComponent(url);
        me.openWebsiteURL(ticketAuthenUrl);
    }

    function error() {
        console.log("error requesting authen ticket");
        //TODO: notify user?
    }

};


CFService.prototype.openMyEProfile = function () {
    var profile = this.getUser().getProfileLink();

    this.openEProfile(profile);
};

CFService.prototype.openEProfile = function (profileLink) {

    var url = this.baseUrl + '/members/profile/' + encodeURIComponent(profileLink);

    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openMyProjects = function () {
    var url = this.baseUrl + '/members/project-overview';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openCastingLetter = function () {
    var url = this.baseUrl + '/castingletter';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openOnlineHelp = function () {
    var url = this.baseUrl + '/members/kb';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openFullVersion = function () {
    var url = this.baseUrl + '/members/home';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openESearch = function () {
    var url = this.baseUrl + '/members/e-search/actor';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openECastings = function () {
    var url = this.baseUrl + '/members/e-casting/own';
    this.openWebsiteUrlWithAuthentication(url)
};

CFService.prototype.openPasswordReset = function () {
    var url = this.baseUrl + '/members/reset-password';
    this.openWebsiteURL(url)
};

CFService.prototype.openRegisterPage = function () {
    var url = this.baseUrl + '/app-redirect-registration.php';
    this.openWebsiteURL(url)
};


CFService.prototype.getAvailableLanguageIds = function () {
    return this.availableLanguageIds;
};

CFService.prototype.getVideoListByProfileId = function (profileId) {
    var def = this.q.defer();
    this.retrieveStruct('videos-' + profileId + '.json').then(
        function (list) {
            if (!list) {
                list = [];
            }
            def.resolve(list);
        },
        function (error) {
            def.reject(error);
        }
    );
    return def.promise;
};

CFService.prototype.setVideoListForProfileId = function (profileId, videoList) {
    var p = this.storeStruct('videos-' + profileId + '.json', videoList);
    return p;
};

CFService.prototype.hasVideoListForProfileId = function (profileId) {
    return this.hasTextObject('videos-' + profileId + '.json');
};

CFService.prototype.getPictureListByProfileId = function (profileId) {
    var def = this.q.defer();
    this.retrieveStruct('pictures-' + profileId + '.json').then(
        function (list) {
            if (!list) {
                list = [];
            }
            def.resolve(list);
        },
        function (error) {
            def.reject(error);
        }
    );
    return def.promise;
};

CFService.prototype.setPictureListForProfileId = function (profileId, picList) {
    var p = this.storeStruct('pictures-' + profileId + '.json', picList);
    return p;
};

CFService.prototype.hasPictureListForProfileId = function (profileId) {
    return this.hasTextObject('pictures-' + profileId + '.json');
};


CFService.prototype.getClientList = function () {
    var def = this.q.defer();
    this.retrieveStruct('clients.json').then(
        function (list) {
            if (!list) {
                list = [];
            }
            def.resolve(list);
        },
        function (error) {
            def.reject(error);
        }
    );
    return def.promise;
};

CFService.prototype.setClientList = function (clientList) {
    var p = this.storeStruct('clients.json', clientList);
    return p;
};

CFService.prototype.hasClientList = function () {
    return this.hasTextObject('clients.json');
};


CFService.prototype.isOffline = function () {
    return this.offline;
};

CFService.prototype.isOnline = function () {
    return !this.offline;
};

CFService.prototype.setOnline = function () {
    console.log("set to online");
    this.offline = false;
};

CFService.prototype.setOffline = function () {
    console.log("set to offline");
    this.offline = true;
};

CFService.prototype.initStorage = function () {
    var me = this;
    var url = this.getStorageBaseUrl();
    console.log("data dir=" + url);

    this.useFS.fs().useDir(url).createSubDir('data').then(function (dirEntry) {
        console.log("created data dir", dirEntry);
        me.dataDirUrl = dirEntry.toURL();

    }, function () {
        console.log("failed to create data dir");
    });
};

CFService.prototype.initPrivateStorage = function () {
    var me = this;
    var url = this.getPrivateStorageBaseUrl();
    console.log("private data dir=" + url);

    this.useFS.fs().useDir(url).createSubDir('data').then(function (dirEntry) {
        console.log("created private data dir", dirEntry);
        me.privateDataDirUrl = dirEntry.toURL();
    }, function () {
        console.log("failed to create private data dir");
    });
};


CFService.prototype.getDataDirUrl = function () {
    return this.dataDirUrl;
};

CFService.prototype.storeTextObject = function (key, data, appPrivate) {
    //console.log("storing text: key=" + key + ", data=" + data);
    var url = this.getStorageBaseUrl();
    if (appPrivate) {
        url = this.getPrivateStorageBaseUrl();
    }
    return this.useFS.fs().useDir(url).useSubDir('data').useFile(key).writeText(data);
};

CFService.prototype.retrieveTextObject = function (key, appPrivate) {
    var url = this.getStorageBaseUrl();
    if (appPrivate) {
        url = this.getPrivateStorageBaseUrl();
    }
    var promise = this.useFS.fs().useDir(url).useSubDir('data').useFile(key).readText();
    return promise;
};

CFService.prototype.removeObject = function (key, appPrivate) {
    var url = this.getStorageBaseUrl();
    if (appPrivate) {
        url = this.getPrivateStorageBaseUrl();
    }
    var promise = this.useFS.fs().useDir(url).useSubDir('data').useFile(key).unlink();
    return promise;
};

CFService.prototype.storeStruct = function (key, jsStruct, appPrivate) {
    var asText = JSON.stringify(jsStruct);
    var p = this.storeTextObject(key, asText, appPrivate);
    return p;
};

CFService.prototype.retrieveStruct = function (key, appPrivate) {
    var me = this;

    var def = this.q.defer();

    this.retrieveTextObject(key, appPrivate).then(success, fail);

    function success(textData) {
        if (!textData) {
            def.resolve(null);
            return;
        }
        var jsStruct = JSON.parse(textData);
        def.resolve(jsStruct);
    }

    function fail(error) {
        def.reject(error);
    }

    return def.promise;
};

CFService.prototype.hasTextObject = function (key, appPrivate) {
    var url = this.getStorageBaseUrl();
    if (appPrivate) {
        url = this.getPrivateStorageBaseUrl();
    }
    var promise = this.useFS.fs().useDir(url).useSubDir('data').exists(key);
    return promise;
};

CFService.prototype.deleteAllData = function () {
    var me = this;
    var url = this.getStorageBaseUrl();

    me.videoDownloader.cancelDownloads(true);
    me.pictureDownloader.cancelDownloads(true);

    var promise = this.useFS.fs().useDir(url).useSubDir('data').removeAllFiles().then(function () {


        url = me.getPrivateStorageBaseUrl();
        return me.useFS.fs().useDir(url).useSubDir('data').removeAllFiles();

    });


    return promise;
};

CFService.prototype.getStorageBaseUrl = function () {
    // android and ios specific
    var platform = this.deviceService.getPlatform();
    var baseUrl = '';
    if (platform === 'Android') {
        // first try SD Card, then internal storage
        if (cordova.file.externalDataDirectory) {
            // maps to /SDCARD/Android/data/APP_PACKAGE/files
            baseUrl = cordova.file.externalDataDirectory
        } else {
            // maps to /data/data/APP_PACKAGE/files
            baseUrl = cordova.file.dataDirectory;
        }
    } else if (platform === 'iOS') {
        // maps to /var/mobile/Applications/UUID/Library/NoCloud
        baseUrl = cordova.file.dataDirectory;
    } else {
        throw 'unsupported platform';
    }

    baseUrl = baseUrl.replace(/\/$/, '');

    // iOS6 quirks: cordova.file.dataDirectory ist mit file://localhost/... was dann einen FileError.ENCODING_ERROR verursacht
    baseUrl = baseUrl.replace(/file:\/\/localhost\//, 'file:///');

    return baseUrl;
};

CFService.prototype.getPrivateStorageBaseUrl = function () {
    var baseUrl = cordova.file.dataDirectory;
    // maps to /data/data/APP_PACKAGE/files on android and
    // maps to /var/mobile/Applications/UUID/Library/NoCloud

    // both dirs a app private

    baseUrl = baseUrl.replace(/\/$/, '');
    // iOS6 quirks: cordova.file.dataDirectory ist mit file://localhost/... was dann einen FileError.ENCODING_ERROR verursacht
    baseUrl = baseUrl.replace(/file:\/\/localhost\//, 'file:///');

    return baseUrl;
};

CFService.prototype.getVideoStreamUrl = function (videoId, auth) {
    var url = this.cfClient.getBaseUrl() + '/gateway/stream.php?id=' + encodeURIComponent(videoId) + '&auth=' + encodeURIComponent(auth) + '&quality=' + encodeURIComponent(VIDEO_QUALITY_MOBILE_MP4);
    return url;
};

CFService.prototype.getVideoDownloadUrl = function (videoId, auth, width) {
    var quality = VIDEO_QUALITY_DVD_MP4;
    if (width >= 1280) {
        quality = VIDEO_QUALITY_HD_MP4;
    }
    var url = this.cfClient.getBaseUrl() + '/gateway/stream.php?id=' + encodeURIComponent(videoId) + '&auth=' + encodeURIComponent(auth) + '&quality=' + encodeURIComponent(quality);
    return url;
};

CFService.prototype.getPictureDownloadUrl = function (fotoId, auth, height) {
    var url = this.cfClient.getBaseUrl() + '/gateway/getFoto.php?id=' + encodeURIComponent(fotoId) + '&auth=' + encodeURIComponent(auth);
    if (height) {
        url = url + '&height=' + encodeURIComponent(height);
    }
    return url;
};

CFService.prototype.getClientPictureUrl = function (profileId) {
    var url = this.cfClient.getBaseUrl() + "/gateway/getFoto.php?id=" + encodeURIComponent(profileId)
};


CFService.prototype.isVideoDownloaded = function (videoId) {
    var url = this.getStorageBaseUrl();
    var promise = this.useFS.fs().useDir(url).useSubDir('data').exists('video-' + videoId + '.mp4');

    return promise;
};

CFService.prototype.isPictureDownloaded = function (fotoId) {
    var url = this.getStorageBaseUrl();
    var promise = this.useFS.fs().useDir(url).useSubDir('data').exists('picture-' + fotoId + '.jpg');

    return promise;
};

CFService.prototype.getOrientation = function () {
    var w = window.innerWidth;
    var h = window.innerHeight;
    if (w > h) {
        return ORIENTATION_LANDSCAPE;
    }
    return ORIENTATION_PORTRAIT;
};


CFService.prototype.getVideoDownloader = function () {
    return this.videoDownloader;
};

CFService.prototype.getPictureDownloader = function () {
    return this.pictureDownloader;
};

CFService.prototype.downloadVideos = function (playlist) {
    for (var i = 0; i < playlist.length; i++) {
        var video = playlist[i];

        var dlConfig = {
            sourceUrl: this.getVideoDownloadUrl(video.videoId, video.auth, video.width),
            targetPath: this.dataDirUrl + '/video-' + video.videoId + ".mp4",
            data: video,
            options: {}
        };
        var dlRef = this.videoDownloader.queueDownload(dlConfig);
        console.log("queued dl", dlRef, dlConfig);
    }
};

CFService.prototype.isProfilePictureDownloaded = function (videoId) {
    var url = this.getStorageBaseUrl();
    var promise = this.useFS.fs().useDir(url).useSubDir('data').exists('profilepicture-' + videoId + '.jpg');

    return promise;
};

CFService.prototype.downloadProfilePictures = function (missing, successCB, failureCB) {
    var me = this;

    var maxHeight = 120;

    if (window.devicePixelRatio > 1) {
        // support retina und so
        maxHeight = maxHeight * window.devicePixelRatio;
    }

    for (var i = 0; i < missing.length; i++) {
        (function () {
            // extra scope for promise handler
            var pic = missing[i];

            var dlConfig = {
                sourceUrl: me.getPictureDownloadUrl(pic.fotoId, pic.auth, maxHeight),
                targetPath: me.getDataDirUrl() + '/profilepicture-' + pic.fotoId + ".jpg",
                data: pic,
                options: {},
                defer: me.q.defer()
            };

            dlConfig.defer.promise.then(function (fileEntry) {
                if (successCB) {
                    successCB(fileEntry, pic);
                }
            }, function (error) {
                if (failureCB) {
                    failureCB(error, pic);
                }
            });
            var dlRef = me.getPictureDownloader().queueDownload(dlConfig);
            //console.log("queued dl", dlRef, dlConfig);
        })();
    }
};


CFService.prototype.forgetUser = function () {
    return this.removeObject('user.json', true);
};

CFService.prototype.saveUser = function () {

    var info = this.user.getInitConfig();

    return this.storeStruct('user.json', info, true);
};

CFService.prototype.resurrectUser = function () {
    var me = this;

    return this.hasTextObject('user.json', true).then(function () {
        return me.retrieveStruct('user.json', true).then(function (info) {
            if (info === null) {
                throw "no user info persisted";
            }
            var user = new CFUser(info);
            return user;
        });
    });

};

CFService.prototype.getAppVersion = function () {
    return APP_VERSION;
};

CFService.prototype.downloadTranslationUpdates = function () {
    var me = this;

    for (var i = 0; i < this.availableLanguageIds.length; i++) {
        (function () {
            var lngKey = me.availableLanguageIds[i];
            var promise = me.cfClient.getLanguageUpdate(lngKey);
            promise.then(
                function (response) {
                    me.storeTextObject('translation_' + lngKey + '.json', response, true);
                    // auch gleich aktivieren
                    me.activateTranslation(lngKey, response);
                },
                function () {
                    console.log("translation download failed, use local version");
                    me.activateTranslationUpdate(lngKey);
                }
            )
        })();
    }
};

CFService.prototype.activateTranslation = function (lngKey, texte) {

    var json = texte.replace(/^[^\{]*/, '');
    json = json.replace(/[^\}]*$/, '');

    try {
        var trans = JSON.parse(json);

        cfetApp.translations[lngKey] = trans;
        console.log("translation update activated for " + lngKey);
    } catch (x) {
        console.log("translation activation failed for " + lngKey, x);
    }
};


CFService.prototype.activateTranslationUpdate = function (lngKey) {
    var me = this;

    var promise = me.retrieveTextObject('translation_' + lngKey + '.json', true);
    promise.then(
        function (response) {
            me.activateTranslation(lngKey, response);
        },
        function () {
            console.log("no translation update for " + lngKey);
        }
    );
};


CFService.prototype.activateTranslationUpdates = function () {
    var me = this;

    for (var i = 0; i < me.availableLanguageIds.length; i++) {
        var lngKey = me.availableLanguageIds[i];
        me.activateTranslationUpdate(lngKey);
    }
};


