/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var app = {
    // Application Constructor
    initialize: function () {
        app.bindEvents();
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function () {
        console.log("binding events");
        document.addEventListener('deviceready', app.onDeviceReady, false);
        window.addEventListener('orientationchange', app.onOrientationChange, false);
        window.addEventListener('resize', app.onResize, false);
        document.addEventListener('backbutton', app.onBackButton, false);
    },
    // deviceready Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicity call 'app.receivedEvent(...);'
    onDeviceReady: function () {
        console.log("deviceready " + window.location.href);
        StatusBar.hide();

        navigator.splashscreen.hide();

        app.initialWidth = window.innerWidth;
        app.initialHeight = window.innerHeight;
        if (app.initialWidth > app.initialHeight) {
            app.initialOrientation = ORIENTATION_LANDSCAPE;
        } else {
            app.initialOrientation = ORIENTATION_PORTRAIT;
        }
        console.log("w=" + app.initialWidth + " h=" + app.initialHeight + " o=" + app.initialOrientation);

        var w = window.innerWidth;
        var h = window.innerHeight;

        //$('.fullScreen').height(h);
        //$('.fullScreen').width(w);
        //$('.fullScreenVideo').height(h);
        //$('.fullScreenVideo').width(w);


        //$('.galPic').css({"max-height": (h-100) + 'px'})
        //$('.videoPlayerFullScreenSmartPhone').css({width: w + 'px', height: (h-100) + 'px'});



        function initAngular() {
            // bootstrap angular
            angular.bootstrap(document, ['CFETApp']);
            StatusBar.hide();
        }

        navigator.globalization.getPreferredLanguage(function (languageInfo) {

            var lngId = languageInfo.value;
            if (lngId) {
                // nur die Sprache nehmen
                lngId = lngId.replace(/(-|_).*$/, '');
            }

            app.usersLanguageId = lngId;
            initAngular();
        }, function () {
            // failed to get language from the system, set to de as fallback
            app.usersLanguageId = 'de';
            initAngular();
        });
    },
    onOrientationChange: function () {
        console.log("orientation=" + window.orientation);
        // dieser event kommt etwas spaeter als der resize event
        // fuer den inline videoplayer auf dem showreel stimmen aber dann die berechneten groessen
        //app.onResize();

    },
    onResize: function (info) {

        var w = window.innerWidth;
        var h = window.innerHeight;
        var hSpacerWidth = $('.v-spacer').outerWidth();

        var maxHeight = h;
        var maxWidth = w;

        // fuer rotate der bilder galerie komplette geraetebreite
        $('.maxHeight').css({"height": maxHeight + 'px'});
        $('.maxWidth').css({"width": maxWidth + 'px'});
        var maxContentWidth = maxWidth - (2 * hSpacerWidth);
        $('.maxContentWidth').css({"width": maxContentWidth + 'px'});

        // video showreelplayer


        if ($('#v1').length == 1) {
            var videoWidth = $('#videoplayercontainer').width()
            var videoHeight = $('#videoplayercontainer').height();
            console.log("vW=" + videoWidth + ', vH=' + videoHeight);
            if (h > w) {
                // bei hoch stimmt der rand nicht mehr
                videoWidth = maxContentWidth;
                videoHeight = videoHeight - hSpacerWidth - hSpacerWidth; // 2mal wg. padding-top
            }
            console.log("2 vW=" + videoWidth + ', vH=' + videoHeight);

            $('#v1').width(videoWidth);
            $('#v1').height(videoHeight);

            console.log($('#v1')[0]);

            var mp = app.getMediaPlayer();
            mp.setPlayerSize(videoWidth, videoHeight);
            $('#v1').width(videoWidth);
            $('#v1').height(videoHeight);

        } else {
            console.log("no vid resize tablet");
        }


        // video smartphone
        if ($('#v2').length == 1) {

            // fullscreen aktiv?
            if ($('.mejs-container-fullscreen').length == 1) {

                console.log("rotating fullscreen video");

            } else {

                var videoWidth = $('#videoPlayerFull').width()
                var videoHeight = $('#videoPlayerRow').height();
                console.log("vW=" + videoWidth + ', vH=' + videoHeight);
                console.log("2 vW=" + videoWidth + ', vH=' + videoHeight);

                $('#v2').width(videoWidth);
                $('#v2').height(videoHeight);
                var mp = app.getMediaPlayer();
                mp.setPlayerSize(videoWidth, videoHeight);
                $('#v2').width(videoWidth);
                $('#v2').height(videoHeight);
            }
        }

    },
    onBackButton: function () {
        console.log('back nav from: ' + window.location.hash);
        if (window.location.hash.match(/startseite|login/)) {
            console.log("exiting app");
            navigator.app.exitApp();
        } else {
            navigator.app.backHistory();
        }
    },
    setMediaPlayer: function (mediaPlayer) {
        app.__mediaPlayer = mediaPlayer;
    },
    getMediaPlayer: function() {
        return app.__mediaPlayer;
    }

};


// kick off app
app.initialize();